@@ -31,6 +31,18 @@ from enrichapp.spec import validate_spec as default_validate_spec, fill_action_g
 from enrichsdk.lib import get_credentials_by_name
 from enrichsdk.utils import SafeEncoder
 from enrichsdk.datasets import Doodle
+from enrichsdk.lib.integration import send_html_email
+from enrichsdk.lib import get_default_from_email
+
+from email import encoders
+from email.mime.multipart import MIMEMultipart
+from email.mime.text import MIMEText
+from email.mime.audio import MIMEAudio
+from email.mime.base import MIMEBase
+from email.mime.image import MIMEImage
+
+import pandas as pd
+from pretty_html_table import build_table
 
 logger= logging.getLogger('app')
 
@@ -53,9 +65,9 @@ class AppBase():
             'policy_select_dataset': f'enrichapp/policyapp/policy_select_dataset.html',
             'policy_select_columns': f'enrichapp/policyapp/policy_select_columns.html',
             'policy_select_columns_helper': f'enrichapp/policyapp/policy_select_columns_helper.html',
-
             'result_index': f'enrichapp/policyapp/result_index.html',
             'result_detail': f'enrichapp/policyapp/result_detail.html',
+            'workflow_index': f'enrichapp/policyapp/workflow_index.html',
 
         }
         self.select_columns_post_spec = {
@@ -65,6 +77,9 @@ class AppBase():
     def __str__(self):
         return f"[{self.name}] {self.verbose_name}"
 
+    def get_readme(self):
+        return """This is a generic policy app"""
+
     def get_template(self, spec, name):
         return spec.get('templates',{}).get(name, self.templates[name])
 
@@ -137,7 +152,9 @@ class AppBase():
             url('^configurations/select/dataset[/]?$', self.policy_select_dataset, name="policy_select_dataset"),
             url('^configurations/select/columns[/]?$', self.policy_select_columns, name="policy_select_columns"),
             url('^results[/]?$', self.result_index, name="result_index"),
-            url('^results/detail[/]?$', self.result_detail, name="result_detail")
+            url('^results/detail[/]?$', self.result_detail, name="result_detail"),
+            url('^workflows/action[/]?$', self.workflow_action, name="workflow_action"),
+            url('^workflows[/]?$', self.workflow_index, name="workflow_index")
         ]
 
         return urlpatterns
@@ -176,11 +193,11 @@ class AppBase():
             policies = PolicyModel.objects\
                                   .filter(appname=self.name, namespace=namespace)\
                                   .order_by("-id")
+
         except:
             logger.exception(f"Unable to get {self.name} policies. Please see server log")
             #return HttpResponseRedirect(reverse(r.namespace + ":index"))
 
-
         return render(request,
                       template,
                       {
@@ -452,30 +469,55 @@ class AppBase():
     def get_result_list(self, request, spec, policies):
 
         try:
-            available = []
+            available = {}
 
             data = spec['data']
             handle = self.get_fs_handle(data['cred'])
 
             appname = self.name
             namespace = spec['namespace']
+
             root = os.path.join(data['root'], appname, namespace)
 
-            # available paths
-            # Look for the subset within the namespace
-            configurations = handle.ls(root)
-            configurations = [os.path.basename(c) for c in configurations]
+            fallback = spec.get('fallback',False)
+            if fallback:
+                if not handle.exists(root):
+                    messages.warning(request, "Result root directory is missing. choosing alternative")
+                    namespaces = handle.ls(os.path.join(data['root'], appname))
+                    if len(namespaces) == 0:
+                        return []
+                    root = namespaces[0]
+
 
             # What should I be looking at?
             relevant_configurations = [p.name for p in policies if ((p.appname == self.name) &
                                                                     (p.namespace == namespace))]
 
-            available = [c for c in configurations if ((c in relevant_configurations) or True)]
+            # available paths
+            # Look for the subset within the namespace
+            paths = handle.find(root, maxdepth=2, withdirs=True)
+            confmap = {}
+            for p in paths:
+                p = p.replace(root+ "/", "").split("/")
+                if len(p) != 2:
+                    continue
+                confname, dt = p
+                if confname not in confmap:
+                    confmap[confname] = {
+                        'dates': []
+                    }
+                confmap[confname]['dates'].append(dt)
+
+            for confname in list(confmap.keys()):
+                confmap[confname]['latest'] = max(confmap[confname]['dates'])
+
+            available = {c: detail for c, detail in confmap.items() if ((c in relevant_configurations) or True)}
 
             if hasattr(handle, 'close'):
                 handle.close()
 
         except:
+            #traceback.print_exc()
             logger.exception(f"Error in accessing {spec['name']} results")
 
         return available
@@ -500,12 +542,22 @@ class AppBase():
 
             appname = self.name
             namespace = spec['namespace']
-            name = policy.name
+
             root = os.path.join(data['root'], appname, namespace)
+
+            # may be different from policy.name
+            name = request.GET.get('name')
+
+            fallback = spec.get('fallback',False)
+            if fallback:
+                if not handle.exists(root):
+                    messages.warning(request, "Result root directory is missing. choosing alternative")
+                    namespaces = handle.ls(os.path.join(data['root'], appname))
+                    root = namespaces[0]
+
             names = [os.path.basename(n) for n in handle.ls(root)]
-            if policy.name not in names:
-                raise Exception(f"Could not find {policy.name}")
-            name = policy.name
+            if name not in names:
+                raise Exception(f"Could not find {name}")
 
             dates = sorted(handle.ls(os.path.join(root, name)), reverse=True)
             if len(dates) == 0:
@@ -582,6 +634,7 @@ class AppBase():
         """
         Show the available results
         """
+        from dashboard.models import AppActionTarget
 
         r = resolve(request.path)
 
@@ -589,11 +642,12 @@ class AppBase():
         if not valid:
             return redirect
 
+        fallback = spec.get('fallback', False)
         usecase = spec['usecase']
 
         template = self.get_template(spec, 'result_detail')
 
-        name = request.GET.get('c', None)
+        name = request.GET.get('name', None)
         if name is None:
             messages.error(request, "Missing configuration")
             return HttpResponseRedirect(reverse(r.namespace + ":result_index"))
@@ -608,10 +662,17 @@ class AppBase():
                                                active=True
                                           )
         except Exception as e:
-            policy = PolicyModel.objects.filter(appname=self.name,namespace=namespace, active=True).first()
-            messages.error(request, f"Unable to get configuration for {name}")
             logger.exception(f"Unable to get configuration for {name}")
-            #return HttpResponseRedirect(reverse(r.namespace + ":result_index"))
+            if not fallback:
+                messages.error(request, f"Unable to get configuration for {name}")
+                return HttpResponseRedirect(reverse(r.namespace + ":result_index"))
+
+            messages.warning(request, f"Unable to get configuration for {name}")
+            try:
+                policy = PolicyModel.objects.filter(appname=self.name,namespace=namespace, active=True).first()
+            except:
+                messages.warning(request, f"No policy defined for fallback")
+                return HttpResponseRedirect(reverse(r.namespace + ":result_index"))
 
         path, data = self.get_result(request, spec, policy)
         if data is None:
@@ -619,6 +680,10 @@ class AppBase():
             logger.exception(f"Unable to get data for configuration {name}")
             return HttpResponseRedirect(reverse(r.namespace + ":result_index"))
 
+        # Workflow targets
+        supported_actions = self.get_supported_actions(request, spec, policy)
+        targets = AppActionTarget.objects.filter(nature__in=supported_actions)
+
         return render(request,
                       template,
                       {
@@ -626,11 +691,173 @@ class AppBase():
                           'spec': spec,
                           'usecase': usecase,
                           'basenamespace': r.namespace,
+                          'name': name,
                           'policy': policy,
                           'data': data,
-                          'path': path
+                          'path': path,
+                          'targets': targets
                       })
 
+    ################################################################
+    # Workflow Management
+    ################################################################
+    def get_supported_actions(self, request, spec, policy):
+        return ['Email']
+
+    def take_action(self, request, spec, name, data, target):
+
+        r = resolve(request.path)
+        detailurl = request.build_absolute_uri(reverse(r.namespace + ":result_detail")) + f"?name={name}"
+
+        try:
+            if target.nature == "Email":
+
+                """{
+                    "emails": [
+                       "pingali@scribbledata.io"
+                     ]
+                }"""
+
+                policy = target.policy
+                if (('emails' not in policy) and (not isinstance(target.policy['emails'], list))):
+                    return JsonResponse({
+                        'status': 'failure',
+                        'message': "Email service not configured correct. Please contact the admin"
+                    })
+
+                user = request.user
+                fullname = f"{user.first_name} {user.last_name}".strip()
+                if len(fullname) == 0:
+                    fullname = user.username
+
+                sender = fullname + "<" + get_default_from_email() +">"
+                receivers = policy['emails']
+                subject = f"{spec['name']} - Sharing Observation"
+                content = f"<p>{data['message']}</p>"
+                content += f"<p>You can find the full details at <a href='{detailurl}'>{spec['name']}</a> page. Here is quick context:</p>"
+                table = json.loads(data['data'])
+                s = pd.Series(table).reset_index()
+                s.columns = ['Attribute', 'Value']
+                content += build_table(s,
+                                       'blue_light',
+                                       font_size='small',
+                                       font_family='Roboto, sans-serif')
+
+                msg = MIMEMultipart()
+                msg.attach(MIMEText(content, "html"))
+                send_html_email(content=content,
+                                sender=sender,
+                                receivers=receivers,
+                                subject=subject)
+
+                return JsonResponse({
+                    'status': 'success',
+                    'message': "Email sent successfully"
+                })
+
+            return JsonResponse({
+                'status': "failure",
+                "message": f"Workflow service not configured correctly. Please contact admin"
+            })
+
+        except Exception as e:
+            logger.exception("Unable to send email")
+            return JsonResponse({
+                'status': "failure",
+                "message": f"{target.nature} message action could not be taken. {str(e)}"
+            })
+
+    @log_activity("workflow_action", nature="application")
+    def workflow_action(self, request, spec):
+        """
+        Show the workflows for the given data
+        """
+
+        from dashboard.models import AppActionTarget
+
+        r = resolve(request.path)
+
+        valid, redirect = self.check_prerequisites(request, spec)
+        if not valid:
+            return redirect
+
+        usecase = spec['usecase']
+
+        name = request.GET.get('name', None)
+        if name is None:
+            logger.exception("Missing name of the result")
+            return HttpResponseRedirect(reverse(r.namespace + ":result_index"))
+
+        data = request.POST.dict()
+
+        try:
+            try:
+                target = AppActionTarget.objects.get(pk=data['actionid'])
+            except:
+                raise Exception("Internal error. Missing or invalid action id")
+
+            messages.success(request, "Processed the post")
+            status = "success"
+
+            # Make this happen...
+            status = self.take_action(request, spec, name, data, target)
+
+        except Exception as e:
+            return JsonResponse({
+                'status': 'failure',
+                'message': f"Failed to trigger action. {str(e)}"
+            })
+
+        return status
+
+    @log_activity("workflow_index", nature="application")
+    def workflow_index(self, request, spec):
+        """
+        Show the workflows for the given data
+        """
+
+        from dashboard.models import AppActionTarget
+
+
+        r = resolve(request.path)
+
+        valid, redirect = self.check_prerequisites(request, spec)
+        if not valid:
+            return redirect
+
+        usecase = spec['usecase']
+
+        name = request.GET.get('name', None)
+        if name is None:
+            logger.exception("Missing name of the result")
+            return HttpResponseRedirect(reverse(r.namespace + ":result_index"))
+
+        data = request.GET.get('data', '{}')
+        try:
+            data = json.loads(data)
+        except:
+            logger.exception("Missing data dictionary required for workflows")
+            return HttpResponseRedirect(reverse(r.namespace + ":result_index"))
+
+        template = self.get_template(spec, 'workflow_index')
+
+        targets = AppActionTarget.objects.filter(nature__in=self.get_supported_actions(request, spec, policy))
+
+        return render(request,
+                      template,
+                      {
+                          'app': self,
+                          'spec': spec,
+                          'usecase': usecase,
+                          'basenamespace': r.namespace,
+                          "name": name,
+                          'data': data,
+                          'targets': targets
+                      })
+
+    ################################################################
+    # API
+    ################################################################
     @log_activity("app_policy_index_api", nature="application")
     def api_policy_index(self, request, instancename):
         """
@@ -660,6 +887,9 @@ class AppBase():
         }
 
 
+    ################################################################
+    # Ninja Routing/URLS setup
+    ################################################################
     def get_router(self):
 
         from ninja import Router, Query, Schema
