@@ -91,6 +91,12 @@ class DataQualityBase(Compute):
         config = spec['config']
         expectations = config['expectations']
 
+        # what is the expected result format?
+        result_format = config.get('result_format', 'none')
+        if result_format not in ["basic", "complete", "boolean_only", "summary"]:
+            result_format = "basic"
+        result_format = result_format.upper()
+
         tested_at = datetime.now()
 
         run_name = f"{name}-{tested_at}"
@@ -117,7 +123,9 @@ class DataQualityBase(Compute):
             if do_test is False:
                 continue
 
+            # set some params explicitly for the call to GE
             expectation['parameters']['column'] = expectation['column']
+            expectation['parameters']['result_format'] = result_format
 
             # now we can run the expectation test
             if expectation.get('type', 'builtin') == 'builtin':
