@@ -506,12 +506,28 @@ class InMemoryQueryExecutorBase(AnonymizerMixin, Compute):
                     msg += "Columns: {}\n".format(", ".join(df.columns))
                     msg += "Dtypes: " + df.dtypes.to_string() + "\n"
 
+                    skip_empty = spec.get("skip_empty", True)
                     if len(df) == 0:
                         # no data returned...
-                        logger.warning(
-                            f"Completed {name} {params['start_date']} No data",
-                            extra={"transform": self.name, "data": msg},
-                        )
+                        if skip_empty:
+                            logger.warning(
+                                f"Completed {name} {params['start_date']} No data",
+                                extra={"transform": self.name, "data": msg},
+                            )
+                            continue
+                        else:
+                            logger.warning(
+                                f"{name} {params['start_date']} No data",
+                                extra={"transform": self.name, "data": msg},
+                            )
+
+                    if ((len(df) == 0) and (not callable(segmentcol))):
+                        msg = """Dont know how to handle an empty dataframe. Not sure what columns should be included with what values. segmentcol should be a callable""" + msg
+                        logger.warning(f"{name} {params['start_date']} Skipping",
+                                       extra={
+                                           "transform": self.name,
+                                           "data": msg
+                                       })
                         continue
 
                     # First gather a map of segments
