@@ -323,7 +323,7 @@ class MetricsBase(Compute):
             if isinstance(result, dict):
                 datasets.update(result)
             else:
-                datasets[name] = df
+                datasets[name] = result
 
         missing = [s for s in required_sources if s not in found]
         if len(missing) > 0:
@@ -493,7 +493,7 @@ class MetricsBase(Compute):
 
         if (not hasattr(self, "read_data")) or (not hasattr(self, "get_dataset")):
             raise Exception(
-                " get_dataset_raw expects read_data and get_dataset methods"
+                " get_dataset_generic expects read_data and get_dataset methods"
             )
 
         args = self.args
