@@ -7,6 +7,7 @@ import random
 import sqlite3
 import tempfile
 import hashlib
+import shutil
 from enrichsdk import Compute, S3Mixin
 from datetime import datetime, timedelta, date
 import logging
@@ -593,6 +594,9 @@ class DataObserverBase(Compute):
         # notification via email
         name        = spec['name']
         receivers   = notify_obj['receivers']
+        bcc         = notify_obj.get('bcc', [])
+        cc         = notify_obj.get('cc', [])
+        reply_to    = notify_obj.get('reply_to', [])
         content     = notify_obj.get('content')
         subject     = notify_obj.get('subject')
         sender      = notify_obj.get('sender')
@@ -609,40 +613,62 @@ class DataObserverBase(Compute):
             content = f"[{self.args['run_date']}] Ledger Balance Alert"
 
         # write the result to tmp
-        tmpdir = tempfile.mkdtemp()
-        fname = os.path.join(tmpdir, f'LEDGER-BALANCE-CHECK_{name}.csv')
-        data.to_csv(fname, index=False)
-
-        if send_attachment:
-            attachments = [fname]
-        else:
-            attachments = []
-
-        # send the email
         try:
-            notify_success = True
-            msg = f"Spec: {name}" + "\n"
-            msg += f"Sender: {sender}" + "\n"
-            msg += f"Receivers: {receivers}" + "\n"
-            msg += f"Subject: {subject}" + "\n"
-            msg += f"Content: {content}" + "\n"
+            msg = ""
+            tmpdir = tempfile.mkdtemp()
+            fname = os.path.join(tmpdir, f'LEDGER-BALANCE-CHECK_{name}.csv')
+            data.to_csv(fname, index=False)
 
-            send_html_email(content,
-                            sender=sender,
-                            receivers=receivers,
-                            subject=subject,
-                            attachments=attachments,
-                            cred=smtpcred)
-            logger.debug(
-                f"Sent notifications",
-                extra={"transform": self.name, "data": msg}
-            )
+            if send_attachment:
+                attachments = [fname]
+            else:
+                attachments = []
+
+            # send the email
+            try:
+                notify_success = True
+                msg = f"Spec: {name}" + "\n"
+                msg += f"Sender: {sender}" + "\n"
+                msg += f"Receivers: {receivers}" + "\n"
+                msg += f"Reply-To: {reply_to}" + "\n"
+                msg += f"CC: {cc}" + "\n"
+                msg += f"BCC: {bcc}" + "\n"
+                msg += f"Subject: {subject}" + "\n"
+                msg += f"Content: {content}" + "\n"
+
+                send_html_email(content,
+                                sender=sender,
+                                receivers=receivers,
+                                cc=cc,
+                                bcc=bcc,
+                                reply_to=reply_to,
+                                subject=subject,
+                                attachments=attachments,
+                                cred=smtpcred)
+                logger.debug(
+                    f"Sent notifications",
+                    extra={"transform": self.name, "data": msg}
+                )
+            except:
+                logger.exception(
+                    f"Error sending email",
+                    extra={"transform": self.name, "data": msg}
+                )
         except:
             logger.exception(
                 f"Error sending email",
                 extra={"transform": self.name, "data": msg}
             )
 
+        if os.path.exists(tmpdir):
+            try:
+                shutil.rmtree(tmpdir)
+            except:
+                logger.exception(
+                    f"Unable to cleanup tmpdir",
+                    extra={"transform": self.name, 'data': tmpdir}
+                )
+
         return notify_success
 
 
