@@ -1087,6 +1087,88 @@ def prepare_prefectjob(path, templatepath):
     # Bye bye
     print("Bootstrapped prefectjob")
 
+def prepare_app_index(path, templatepath):
+    """
+    Bootstrap a new index page for app (using 2.0 format)
+
+    Parameters
+    ----------
+    component: Nature of the module that should be bootstrapped
+    path: Path where the html should be created.
+
+    Yields
+    ------
+    index.html
+
+    """
+
+    if templatepath is not None:
+        raise Exception("app index template not supported yet")
+
+    src = os.path.join(default_templatedir, "appstore2.0","index.html.template")
+    dst = path
+    if os.path.exists(dst):
+        raise Exception("Destination exists. Please remove it")
+
+    # Load the source
+    content = open(src).read()
+
+    # Now get the data and dump it in to the template
+    data = get_appstore_index_data()
+    content = content.replace("DATASTRUCTURE", json.dumps(data, indent=4))
+
+    # Now dump the format
+    with open(dst, 'w') as fd:
+        fd.write(content)
+
+    # Bye bye
+    print("Bootstrapped app html page")
+
+def prepare_singlepageapp(path, templatepath):
+    """
+    Bootstrap a new single page app
+
+    Parameters
+    ----------
+    component: Nature of the module that should be bootstrapped
+    path: Path where the html should be created.
+
+    Yields
+    ------
+    python file
+
+    """
+
+    if templatepath is not None:
+        raise Exception("app index template not supported yet")
+
+    src = os.path.join(default_templatedir, "singlepageapp.py.template")
+    dst = path
+    if os.path.exists(dst):
+        raise Exception("Destination exists. Please remove it")
+
+    # Load the source
+    content = open(src).read()
+
+    # Get the template
+    template = get_template_path(src)
+
+    variables = get_variables(template)
+
+    context = {}
+    for v in variables:
+        context[v] = prompt(
+            "> " + v + ": ",
+        )
+
+    content = render(template, context)
+
+    # Now dump the format
+    with open(dst, 'w') as fd:
+        fd.write(content)
+
+    # Bye bye
+    print(f"Bootstrapped a singlepage app in {dst}")
 
 def bootstrap(component, path, templatepath):
     """
@@ -1158,6 +1240,10 @@ def bootstrap(component, path, templatepath):
         prepare_pyscript(path, templatepath)
     elif component in ["datasets"]:
         prepare_datasets(path, templatepath)
+    elif component in ["app-index"]:
+        prepare_app_index(path, templatepath)
+    elif component in ["singlepageapp"]:
+        prepare_singlepageapp(path, templatepath)
 
 
 def get_sample_siteconf():
