@@ -95,7 +95,7 @@ class MockState(EnrichStateBase):
                 "release": platform.release(),
                 "processor": platform.processor(),
                 "python": platform.python_version(),
-                "distribution": distro.linux_distribution(),
+                "distribution": (distro.id().title(), distro.version(), distro.name())
             },
             "user": user,
         }
@@ -630,6 +630,7 @@ class MockConfig(object):
                 "notes": self.state.notes,
                 "performance": self.state.performance_summary,
                 "stats": self.state.stats,
+                "kpi": self.state.kpi,
                 "start_time": self.state.start_time,
                 "end_time": self.state.end_time,
                 "pid": self.state.pid,
