@@ -127,7 +127,8 @@ def _version():
 
 
 @click.command("start", hidden=True)
-def _start():
+@click.option("--minimal/-m", default=False)
+def _start(minimal):
     """
     Instructions to get started
     """
@@ -302,8 +303,9 @@ def _start():
 
     for i, s in enumerate(steps):
         print("[{:2}] {}".format(i + 1, "\n".join(textwrap.wrap(s["description"]))))
-
         print("")
+        if minimal:
+            continue
         if "cmd" in s:
             if isinstance(s["cmd"], str):
                 cmds = [s["cmd"]]
@@ -364,9 +366,11 @@ def init_(component, path, template, context):
         ["dashboard", "A django app that fits the Enrich Dashboard framework"],
         ["pipeline", "A pipeline configuration"],
         ["prefectjob", "Prefect workflow template"],
+        ["app-index", "Index page of a new app"],
         ["datasets", "Datasets specification file"],
         ["rscript", "R-Script template"],
-        ["pyscript", "A python script template"]
+        ["pyscript", "A python script template"],
+        ["singlepageapp", "Single page app"]
     ]
 
     if component not in [c[0] for c in components]:
@@ -479,7 +483,7 @@ def _dump_versionmap():
 
     timestamp, versionmap = lib.read_versionmap()
     print(json.dumps(versionmap, indent=4))
-    
+
 @envsetup.command("sample-siteconf")
 def _siteconf():
     """
@@ -513,7 +517,7 @@ def _show_cred(name):
     """
     print(json.dumps(lib.get_credentials_by_name(name)))
 
-    
+
 @envsetup.command("populate")
 @click.option("--context", default=None, help="Context file")
 def _populate(context):
@@ -731,14 +735,54 @@ def doodle(ctx, context, cred):
 
 @doodle.command("list-catalogs")
 @click.pass_context
-def _list_soures(ctx):
+def _list_catalogs(ctx):
+    """
+    List catalogs
+    """
+
+    cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
+    doodle = datasets.Doodle(cred)
+    catalogs = doodle.list_catalogs()
+    print(json.dumps(catalogs, indent=4))
+
+@doodle.command("search-catalogs")
+@click.argument("name")
+@click.option("-v", "--version", default="v1")
+@click.pass_context
+def _search_catalogs(ctx, name, version):
+    """
+    Search catalogs
+    """
+
+    cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
+    doodle = datasets.Doodle(cred)
+    catalogs = doodle.search_catalogs(name=name,version=version)
+    print(json.dumps(catalogs, indent=4))
+
+@doodle.command("list-sources")
+@click.pass_context
+def _list_sources(ctx):
     """
     List sources
     """
 
     cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
     doodle = datasets.Doodle(cred)
-    sources = doodle.list_catalogs()
+    sources = doodle.list_sources()
+    print(json.dumps(sources, indent=4))
+
+@doodle.command("search-sources")
+@click.argument("name")
+@click.option("-v", "--version", default="v1")
+@click.pass_context
+def _search_sources(ctx, name, version):
+    """
+    Search sources
+    """
+
+    cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
+    doodle = datasets.Doodle(cred)
+    sources = doodle.search_sources(name=name,version=version)
     print(json.dumps(sources, indent=4))
 
 @doodle.command("show-source")
@@ -753,18 +797,7 @@ def _show_source(ctx, source_id):
     doodle = datasets.Doodle(cred)
     source = doodle.get_source(source_id)
     print(json.dumps(source, indent=4))
-    
-@doodle.command("list-sources")
-@click.pass_context
-def _list_soures(ctx):
-    """
-    List sources
-    """
 
-    cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
-    doodle = datasets.Doodle(cred)
-    sources = doodle.list_sources()
-    print(json.dumps(sources, indent=4))
 
 @doodle.command("update-source")
 @click.argument('source_id')
